<?php
/*
Plugin Name: Advanced Category Excluder
Version: 1.0.4
Plugin URI: http://advanced-category-excluder.dev.rain.hu/
Description: With this plugin you can define categories for some parts of the site to be hidden.
Author: Zsolt Lakatos (DjZoNe) 
Author URI: http://djz.hu/
*/ 

global $targets;

$targets = array('is_archive'=>'Archive','is_home'=>'Home','is_feed'=>'RSS Posts','is_comment_feed'=>'RSS Comments','is_search'=>'Search');

foreach ($targets as $key=>$val)
{
    add_option("ace_categories_".$key,'',"Category Excluder for ".$val,true);
}

function ace_disable() {
 remove_filter('posts_join', 'ace_join');
 remove_filter('posts_where', 'ace_where');
 remove_filter('posts_distinct', 'ace_distinct');
}

function ace_where(&$where) {
    global $wpdb, $wp_query, $targets;
   
    if (is_array($targets))
    {
    	foreach ($targets as $key=>$val) 
    	{
    	   if ($wp_query->$key == 1) $filter = $key;    	
    	}
    }

    $cats_to_exclude = get_option("ace_categories_".$filter);

    if ( !empty($filter) && strlen($cats_to_exclude) > 0)
    {
    	if (empty($wpdb->term_taxonomy)) $where .= " AND $wpdb->post2cat.category_id NOT IN (" . $cats_to_exclude . ")";
    	else 
      {
        $where .= " AND NOT EXISTS (";
        $where .= "SELECT * FROM ".$wpdb->term_relationships." JOIN ".$wpdb->term_taxonomy." ON ".$wpdb->term_taxonomy.".term_taxonomy_id = ".$wpdb->term_relationships.".term_taxonomy_id ";
        $where .= "WHERE ".$wpdb->term_relationships.".object_id = wp_posts.ID AND ".$wpdb->term_taxonomy.".term_id IN (" . $cats_to_exclude . ") )";
      }
    }
    
    return $where;   
}

function ace_join(&$join) {
    global $wpdb, $wp_query, $targets;

    if (is_array($targets))
    {
    	foreach ($targets as $key=>$val) 
    	{
    	    if ($wp_query->$key == 1) $filter = $key;
    	}
	    $cats_to_exclude = get_option("ace_categories_".$filter);
    }

    if ( !empty($filter) && strlen($cats_to_exclude) > 0) 
    {
    	if (empty($wpdb->term_relationships))
    	{
    	    if (!preg_match("/post2cat/i",$join)) $join .= " LEFT JOIN $wpdb->post2cat ON $wpdb->post2cat.post_id = $wpdb->posts.ID";
    	}
    	else
    	{
//    	    if (!preg_match("/$wpdb->term_relationships/i",$join)) $join .=" LEFT JOIN $wpdb->term_relationships ON ($wpdb->posts.ID = $wpdb->term_relationships.object_id) ";
//    	    if (!preg_match("/$wpdb->term_taxonomy/i",$join)) $join .=" LEFT JOIN $wpdb->term_taxonomy ON $wpdb->term_taxonomy.term_taxonomy_id = $wpdb->term_relationships.term_taxonomy_id";
    	}
    }

    return $join;
}

function ace_distinct($distinct) {
   global  $wp_query;
   
   // this won't hurt by default.
   
   return "distinct";
}

function ace_adminmenu()
{
    if ( function_exists('add_submenu_page') ) add_submenu_page('plugins.php', __('ACE Dashboard'), __('ACE Dashboard'), 'manage_options', 'ace_options', 'ace_options');
}

function ace_options()
{
    global $wpdb, $targets;

    if ($_POST['submit'])
    // is this form submited?
    {
      if (is_array($targets))
	    { 
        foreach ($targets as $key=>$val)
    	  {
      		$data = $_POST['categories_'.$key];
      		if ( $data != "")
      		// do we have selected categories?
      		{
      		    $c = count($data)-1; $s = "";
      		    foreach ($data as $k=>$v)
      		    {
      			$s.=trim($v); if ($c != $k) $s.=",";
      		    }
      		    update_option("ace_categories_".$key,$s);
      		}
      		// no, so we clean up.
      		else update_option("ace_categories_".$key,"");
      		unset($data);
    	  }
	    }
    }
    
    if ($_POST['empty'])
    {
    	foreach ($targets as $key=>$val)
    	{
    	    update_option("ace_categories_".$key,"");
    	}
    }

    // for ec3 users
    if ($ec3_cat = get_option("ec3_event_category")) $exclude = 'exclude=2,'.$ec3_cat;
    else $exclude = 'exclude=2';

    $categories = get_categories($exclude.'&hide_empty=0');
?>
<form method="post">
    <div class="wrap">
	<h2>Advanced Category Excluder</h2><p>
	<fieldset class="options"><legend>Usage:</legend><p>
	<p>
	    Here are <?php echo count($targets); ?> different sections on the site. You can define section by section which categories would you like to hide.
	</p>
	<?php foreach ($targets as $key=>$val):
	    $_cats = explode(",",get_option("ace_categories_".$key));
	?>	
	<div style="width:20%; float: left;">
	    <p>
    		<label>
    		    <strong><?php echo $val ?></strong>: <br />
    		</label>
		    <?php
		    foreach ($categories as $category)
		    {
		    ?>
  		    <div style="width: 30px; float: left; clear: both">
  			   <input type="checkbox" name="categories_<?php echo $key; ?>[]" value="<?php echo $category->cat_ID; ?>" <?php if (in_array($category->cat_ID,$_cats)) echo "checked"; ?> />
  		    </div>		    
		      <?php echo $category->cat_name; ?><br />
		    <?php
        } 
        reset($categories); ?><br />
	    </p>
	</div>
	<?php endforeach; ?>
	</fieldset>
	<input type="submit" name="submit" value="OK">
	<input type="submit" name="empty" value="Empty">
    </div>

    </form>
<?php
    $wpdb->flush();
}

add_filter('posts_join', 'ace_join');
add_filter('posts_where', 'ace_where');
add_filter('posts_distinct', 'ace_distinct');
add_action('admin_menu', 'ace_adminmenu');
add_action('admin_head', 'ace_disable');
?>
